class RIPS:
    __slots__ = ('SecId', 'PerformanceId', 'Date', 'Deleted', 'Unit_BAS', 'Unit_USD', 'Unit_EUR', 'Unit_GBP',
                 'Unit_CHF', 'Unit_DKK', 'Unit_NOK', 'Unit_SEK', 'Unit_JPY', 'LastUpdate', 'Unit_SGD',
                 'ReturnType', 'Filled', 'Unit_TWD', 'Unit_HKD', 'Unit_MYR', 'Unit_CNY', 'Unit_ILS', 'Unit_INR',
                 'Unit_CAD', 'Unit_KWD', 'Unit_PLN', 'Unit_AUD', 'Unit_THB', 'Unit_KRW', 'Unit_NZD')

    @classmethod
    def from_dict(cls, row: dict):
        obj = cls()
        for k, v in row.items():
            setattr(obj, k, v)
        return obj



import datetime
import logging
import unittest

import yaml

from morningstar.models.instrument import Instrument
from morningstar.morningstar_client import MorningstarClient
from morningstar.provider.morningstar import Morningstar

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

CONFIG_PATH = 'config-morningstar.yml.dist'
CONFIG = yaml.safe_load(open(CONFIG_PATH))['provider']['morningstar']
CONFIG_LIVE = None
try:
    from morningstar.config import config
    CONFIG_LIVE = config.get("provider")['morningstar']
except:
    logger.info("Skipping live tests.")
    pass


class MorningstarClientTest(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.client = MorningstarClient(Morningstar(config=CONFIG))
        if CONFIG_LIVE is not None:
            self.client_live = MorningstarClient(Morningstar(config=CONFIG_LIVE))

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration missing")
    def test_get_traded_currencies(self):
        currencies = self.client_live.get_traded_currencies(instrument="182.1.NESN")
        self.assertEqual(["CHF"], currencies)

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration missing")
    def test_get_prices_by_most_available(self):
        instrument, prices = self.client_live.get_prices_by_most_available(
            isin="CH0038863350",
            currency="CHF",
            start_date="01-01-2019",
            end_date="03-01-2019"
        )
        self.assertEqual(Instrument.from_string("182.1.NESN"), instrument)

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration missing")
    def test_find_instruments_by_isin_and_currency(self):
        instruments = self.client_live.find_instruments_by_isin_and_currency(isin="CH0038863350", currency="CHF")
        self.assertTrue(Instrument.from_string("182.1.NESN") in instruments)

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration missing")
    def test_get_instrument_prices(self):
        prices = self.client_live.get_instrument_prices(
            instrument="182.1.NESN",
            start_date="03-01-2019",
            end_date="04-01-2019"
        )
        self.assertEqual(
            [
                datetime.datetime(2019, 1, 4, 0, 0),
                datetime.datetime(2019, 1, 3, 0, 0)
            ],
            list(prices.keys())
        )

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration missing")
    def test_get_fx_prices(self):
        prices = self.client_live.get_fx_prices(
            base_currency="USD",
            counter_currency="CHF",
            start_date="01-01-2019",
            end_date="03-01-2019"
        )
        self.assertEqual(
            [
                datetime.datetime(2019, 1, 3, 0, 0),
                datetime.datetime(2019, 1, 2, 0, 0),
                datetime.datetime(2019, 1, 1, 0, 0)
            ],
            list(prices.keys())
        )


from morningstar.models.instrument import Instrument


class MSResult:
    def __init__(self, data) -> None:
        self.data = data

    def __repr__(self):
        return str(self.__dict__)

    def get(self, field_name, default=None):
        return self.data.get(field_name, default)

    @staticmethod
    def from_dict(d: dict):
        return MSResult(data=d)

    def has_instrument(self):
        return self.get_instrument() is not None

    def get_instrument(self):
        try:
            instrument = Instrument.from_dict(self.data)
        except KeyError:
            instrument = None
        return instrument



from morningstar.models.ms_result import MSResult

class MSResponse():
    def __init__(self, errors: list, results: list) -> None:
        self.errors = errors
        self.results = results

    @staticmethod
    def from_dict(d: dict):
        quotes = d['quotes']
        errors = quotes.get('error', [])  # optional
        results = quotes['results']
        if results:
            results = [MSResult.from_dict(r) for r in results]
        return MSResponse(errors=errors, results=results)



from morningstar.models.instrument import Instrument


class TSResult:
    def __init__(self, instrument: Instrument, data: dict) -> None:
        self.data = data
        self.instrument = instrument

    @staticmethod
    def from_json(json_obj):
        return TSResult(
            instrument=Instrument(
                exchange=json_obj['exchangeid'],
                security_type=json_obj['type'],
                symbol=json_obj['symbol']
            ),
            data=json_obj['data']
        )


from abc import ABC, abstractmethod


class Provider(ABC):

    @abstractmethod
    def __init__(self, config):
        """
        Args:
            config (dict): dictionary containing configuration parameters
        """
        self.config = config



from morningstar.provider.on_demand import OnDemand
from morningstar.config import config
from morningstar.models.rips import RIPS
from morningstar.spec.on_demand_spec import OnDemandReturnType
from lxml import etree
from typing import Optional


class OnDemandClient():
    xpath_share_performanceid_basecurrency = "/FundShareClass/PerformanceId/Result[./IsBaseCurrency = 'true']/PerformanceId/text()"
    xpath_universe_morningstarid = "/FundShareClassList/ShareClass/Id/text()"

    def __init__(self, provider=None):
        """ Wraps Provider

        Args:
            provider (Provider): provider instance
        """
        if provider is None:
            provider = OnDemand(config=config.get('provider')['ondemand'])
        self.provider = provider

    def login(self):
        """
        Establish a session before using any method of provider
        """
        self.provider.login()

    def get_historical_rips_by_performanceid(self,
                                             performance_id: str,
                                             return_type: OnDemandReturnType,
                                             start_date: str = '',
                                             end_date: str = '') -> [RIPS]:
        """Fetches historical RIPS for the given performance id

            Args:
                performance_id (str): e.g. "0P0000OO1Z"
                return_type (OnDemandReturnType): e.g. TotalReturn
                start_date (str): "%Y-%m-%d"
                end_date (str): "%Y-%m-%d"

            Returns:
                List of RIPS objects, one for each date
        """

        # Download csv data associated with this performance id
        params_historydata = {'ClientId': self.provider.config['clientid'],
                              'DataType': 'rips',
                              'StartDate': start_date,
                              'EndDate': end_date,
                              'PriceTypeId': str(return_type.value),
                              'PerformanceId': performance_id}
        return self.provider.history_data(params=params_historydata)

    def get_historical_rips_by_fundshareclassid(self,
                                                fundshareclass_id: str,
                                                return_type: OnDemandReturnType,
                                                start_date: str = '',
                                                end_date: str = '') -> [RIPS]:
        """Fetches historical RIPS for the fundshare with the given fundshareclass_id using 
            the performance id associated with its base currency

            Note: 
                If no or more than 1 performance id in base currency is found, a ValueError is raised. 

            Args:
                fundshareclass_id (str): e.g. "F00000IRY4"
                return_type (OnDemandReturnType): e.g. TotalReturn
                start_date (str): "%Y-%m-%d"
                end_date (str): "%Y-%m-%d"

            Returns:
                List of RIPS objects, one for each date
        """

        # Use the fundshareclass_id to retrieve the share's xml
        params_sharexml = {'Package': 'EDW', 'IDType': 'FundShareClassId',
                           'ClientId': self.provider.config['clientid'],
                           'Id': fundshareclass_id}

        root_n_share = self.provider.data_output(params=params_sharexml)

        # Read out basecurrency's performance id
        share_performanceid_basecurrency = root_n_share.xpath(
            self.xpath_share_performanceid_basecurrency)
        if len(share_performanceid_basecurrency) == 0:
            raise ValueError("No performance id in basecurrency could be found")
        if len(share_performanceid_basecurrency) > 1:
            raise ValueError(
                "More than 1 performance id in basecurrency was found: " + str(share_performanceid_basecurrency))
        share_performanceid_basecurrency = str(
            share_performanceid_basecurrency[0])

        # Retrieve RIPS using performance id
        return self.get_historical_rips_by_performanceid(performance_id=share_performanceid_basecurrency,
                                                         return_type=return_type,
                                                         start_date=start_date,
                                                         end_date=end_date)

    def get_historical_rips_by_isin(self,
                                    isin: str,
                                    return_type: OnDemandReturnType,
                                    start_date: str = '',
                                    end_date: str = '',
                                    universe: str = '') -> [RIPS]:
        """Fetches historical RIPS for the fundshare with the given ISIN using the performance id in basecurrency

            Note:
                The ISIN is searched in the specified universe to obtain the morningstar id. From the morningstar id,
                the performance id associated with the base currency is used to retrieve the RIPS.
                If the share is obsolete for too long (45d), or not found in the given universe, or if no
                performance id in base currency can be found, a ValueError is raised. 

            Args:
                isin (str): e.g. "IE00B5BMR087"
                return_type (OnDemandReturnType): e.g. TotalReturn
                start_date (str): "%Y-%m-%d"
                end_date (str): "%Y-%m-%d"
                universe (str): e.g. "CHE"

            Returns:
                List of RIPS objects, one for each date
            """

        # Get universe to find Morningstar Id corresponding to isin
        #   no start and end date for search of isin in universe, max length
        data_get_fundshare_universe = {'ClientId': self.provider.config['clientid'],
                                       'ActiveStatus': '',  # obsolete and active, max coverage (last 45d)
                                       'InvestorType': '',  # both, dont care as only to get id
                                       'LegalStructureId': '',  # all types of funds
                                       'ISIN': isin,
                                       'CountryId': universe}
        root_n_universe = self.provider.get_universe(params=data_get_fundshare_universe)

        # read out ms id from universe xml
        #   It is unfortunately possible that a search by ISIN results in the correct fundshare which has
        #   a different ISIN than the one used for searching. In the resulting universe xml, it is also only
        #   listed with this other ISIN and hence cannot be found by subsetting in the xpath with the searched ISIN.
        #   (ie by using [./ISIN = '"+isin+"'] in xpath). Therefore, because only a single share 
        #   should be found, read out directly without searching by ISIN in the xml
        ms_id = root_n_universe.xpath(self.xpath_universe_morningstarid)
        if len(ms_id) == 0:
            raise ValueError(
                'No Morningstar Id could be found for ISIN {}. Maybe the share is dead for too long or in a different universe?'.format(
                    isin))
        if len(ms_id) > 1:
            raise ValueError('More than one Morningstar Id found for ISIN {}.'.format(isin))
        ms_id = str(ms_id[0])

        # Use the Morningstar Id to retrieve the RIPS of the basecurrency
        return self.get_historical_rips_by_fundshareclassid(fundshareclass_id=ms_id,
                                                            return_type=return_type,
                                                            start_date=start_date,
                                                            end_date=end_date)



from enum import Enum


class FieldCode(Enum):
    """ Morningstar Field Codes (Request)

    Morningstar_Morningstar_Web_Services_Specification_v5.0.pdf - Page 68
    """
    Symbol = "H1"
    Exchange = "H2"
    SecurityType = "H3"
    RootSymbol = "S2"
    ListedCurrency = "S9"
    CompanyName = "S12"
    AdditionalCompanyName = "S13"
    ISIN = "S19"
    ContractName = "S31"
    Country = "S34"
    LocalInstrumentCode = "S33"
    PrimaryMIC = "S676"
    WKN = "S722"
    MorningstarPerformanceID = "D2124"
    CUSIP = "S1012"
    SEDOL = "S1013"
    MorningstarIndustry = "S1041"
    MorningstarGroup = "S1042"
    MorningstarSector = "S1043"
    MorningstarID = "S3059"
    VenueFIGICode = "S2995"
    CountryFIGICode = "S1405"
    ShareClassFIGICode = "S1407"
    GlobalIDInvestmentType = "S1735"


class FieldNames(Enum):
    """ Morningstar Field Names (Response)

    Morningstar_Morningstar_Web_Services_Specification_v5.0.pdf - Page 68
    """
    Symbol = "Symbol"
    Exchange = "Exchange"
    SecurityType = "Security Type"
    # RootSymbol = ""
    ListedCurrency = "Listed Currency"
    CompanyName = "Company name"
    # AdditionalCompanyName = ""
    ISIN = "ISIN code"
    # ContractName = ""
    Country = "Country"
    LocalInstrumentCode = "Local instrument cod"
    PrimaryMIC = ""
    # WKN = ""
    MorningstarPerformanceID = "MS Performance ID"
    # CUSIP =
    # SEDOL = ""
    MorningstarIndustry = "Morningstar Industry"
    MorningstarGroup = "Morningstar Group Na"
    MorningstarSector = "Morningstar Sector N"
    MorningstarID = "MS Investment ID (Se"
    VenueFIGICode = "FIGI code"
    CountryFIGICode = "FIGI country code"
    ShareClassFIGICode = "Shareclass-level FIG"
    GlobalIDInvestmentType = "Global ID investment"

    # Additional Fields?
    # "Listing market for t":"182",
    # "Exchange code":"XVTX",


class FilterTag(Enum):
    """ Morningstar filters to be attached to the request

    Morningstar_Morningstar_Web_Services_Specification_v5.0.pdf - Page 67
    """
    Symbol = "symbol"
    Exchange = "exchange"
    SecurityType = "security"
    RootSymbol = "rootsymbol"
    ListedCurrency = "currency"
    CompanyName = "name"
    ISIN = "isin"
    PrimaryMIC = "mic"
    MorningstarPerformanceID = "perfid"
    CUSIP = "cusip"
    SEDOL = "sedol"
    MorningstarID = "investmentID"
    VenueFIGICode = "VenueFigi"
    CountryFIGICode = "CountryFigi"
    ShareClassFIGICode = "ShareClassFigi"
    GlobalIDInvestmentType = "GIDInvestmentType"



import logging
from datetime import datetime
from typing import Tuple, Optional

from morningstar.models.ms_response import MSResponse
from morningstar.config import config
from morningstar.models.instrument import Instrument
from morningstar.provider.morningstar import Morningstar
from morningstar.spec.web_service_spec import FieldNames, FieldCode, FilterTag

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class MorningstarClient():

    def __init__(self, provider=None):
        """ Wraps Provider

        Args:
            provider (Provider): provider instance
        """
        if provider is None:
            provider = Morningstar(config=config.get("provider")['morningstar'])
        self.provider = provider

    def _get_instrument_price_info(self,
                                   instrument: str,
                                   start_date: str,
                                   end_date: str,
                                   bar_type: str = 'dailybar') -> MSResponse:
        return self.provider.index_ts({
            'pricechangeadjusted&instrument': instrument,
            'sdate': start_date,
            'edate': end_date,
            'type': bar_type
        })

    def get_instrument_prices(self,
                              instrument: str,
                              start_date: str,
                              end_date: str,
                              bar_type: str = 'dailybar') -> dict:
        """Fetches prices for a specific instrument

        Args:
            instrument (str): e.g. "126.1.AMZN"
            start_date (str): "%d-%m-%Y"
            end_date (str): "%d-%m-%Y"
            bar_type (str): e.g. "dailybar"

        Returns:
            Dict of timestamps and prices for given asset.
            {
                datetime(2018, 10, 1, 0, 0) : 329.98,
                ...
            }
        """
        response = self._get_instrument_price_info(
            instrument=instrument,
            start_date=start_date,
            end_date=end_date,
            bar_type=bar_type
        )
        historical_price_dict = {}
        if not response.results:
            return historical_price_dict

        hist_prices = response.results[0].data
        for obs in hist_prices:
            timestamp = datetime.strptime(obs['Date Received (GMT)'], '%d-%m-%Y')  # D953
            price = float(obs['Last price'])  # D2
            historical_price_dict[timestamp] = price
        return historical_price_dict

    def get_fx_prices(self,
                      base_currency: str,
                      counter_currency: str,
                      start_date: str,
                      end_date: str,
                      bar_type: str = 'dailybar') -> dict:
        """Fetches FX prices for a currency pair (BASE/COUNTER)

        Args:
            base_currency (str): e.g. "USD"
            counter_currency (str): CHF
            start_date (str): "%d-%m-%Y"
            end_date (str): "%d-%m-%Y"
            bar_type (str): e.g. "dailybar"

        Returns:
            Dict of timestamps and prices FX rates, e.g. 1 BASE = x COUNTER.
            {
                datetime(2018, 10, 1, 0, 0) : 0.98377,
                ...
            }
        """
        if base_currency == counter_currency:
            raise ValueError("Base- and counter currency must be different.")
        instrument = Instrument(
            exchange='245',
            security_type='20',
            symbol='{}{}LITE'.format(base_currency, counter_currency)
        )
        return self.get_instrument_prices(
            instrument=instrument.__str__(),
            start_date=start_date,
            end_date=end_date,
            bar_type=bar_type
        )

    def get_traded_currencies(self, instrument: str) -> [str]:
        """Finds all traded/listed currencies for a specific instrument

        Args:
            instrument (str): e.g. "126.1.AMZN"

        Returns:
            List of currencies, e.g. ["CHF", "USD"]
        """
        response = self.provider.index({
            'instrument': instrument,
            'fields': '{},D204'.format(FieldCode.ListedCurrency.value)
        })
        if not response.results:
            return []

        results = response.results
        traded_currency_field = 'Traded Currency'
        traded_currencies = [x.get(traded_currency_field) for x in results if x.get(traded_currency_field) is not None]
        if traded_currencies:
            return traded_currencies
        logger.info(
            'No traded currency found for instrument {}, attempt use listed currency instead'.format(instrument))
        listed_currencies = [x.get(FieldNames.ListedCurrency.value) for x in results if
                             x.get(FieldNames.ListedCurrency.value) is not None]
        if listed_currencies:
            return listed_currencies
        logger.info('No listed currency found for instrument {}'.format(instrument))
        return []

    def get_prices_by_most_available(self,
                                     isin: str,
                                     currency: str,
                                     start_date: str,
                                     end_date: str,
                                     bar_type: str = 'dailybar') -> Tuple[Optional[Instrument], dict]:
        """Fetches prices for a specific asset from the exchange which provides most results

        Note:
            A search using the ISIN is being executed after which prices are fetched from all returned exchanges which
            list the security.

        Args:
            isin (str): e.g. "US46625H1005"
            currency (str): e.g. "USD"
            start_date (str): "%d-%m-%Y"
            end_date (str): "%d-%m-%Y"
            bar_type (str): e.g. "dailybar"

        Returns:
            Tuple of instrument and a dict of prices for given asset.

            Instrument,
            {
                datetime(2018, 10, 1, 0, 0) : 0.98377,
                ...
            }
        """
        search_response = self.provider.search({'isin': isin})
        max_instrument = None
        max_prices = {}
        if not search_response.results:
            return max_instrument, max_prices
        results = search_response.results
        logger.info('Found {} search results for ISIN {}'.format(len(results), isin))
        max_nr_of_obs = 0
        for result in results:
            if result.has_instrument():
                instrument = result.get_instrument()
                traded_currencies = self.get_traded_currencies(instrument.__str__())
                if currency not in traded_currencies:
                    logger.info('Skip exchange {} as desired asset currency {} is neither traded nor listed'
                                .format(result.get('Exchange'), currency))
                    continue

                hist_prices = self.get_instrument_prices(
                    instrument=instrument.__str__(),
                    start_date=start_date,
                    end_date=end_date,
                    bar_type=bar_type
                )
                if hist_prices:
                    nr_of_prices = len(hist_prices)
                    if nr_of_prices > max_nr_of_obs:
                        max_nr_of_obs = nr_of_prices
                        max_prices = hist_prices
                        max_instrument = instrument
                else:
                    nr_of_prices = 0

                logger.info('Collected {} historical prices for ISIN {} (instrument: {})'
                            .format(nr_of_prices, isin, instrument))
        return max_instrument, max_prices

    def find_instruments_by_isin_and_currency(self, isin: str, currency: str) -> [Instrument]:
        """Finds instruments given an ISIN and currency

        Note:
            In order to determine whether or not the exchange lists the given ISIN in the desired currency,
            the field ListedCurrency is being used.

        Args:
            isin (str): e.g. "US46625H1005"
            currency (str): e.g. "USD"

        Returns:
            List of instruments associated with ISIN and currency
        """
        search_results = self.provider.search({FilterTag.ISIN.value: isin})
        if not search_results.results:
            return []
        search_results = [x for x in search_results.results if x.get(FieldNames.ListedCurrency.value) == currency]
        return [r.get_instrument() for r in search_results if r.has_instrument()]



from typing import Optional

security_types = {
    1: "Not used",
    2: "Stocks",
    3: "Stock and index options",
    4: "Future options",
    5: "Spots",
    6: "Not used",
    7: "Corporate bonds",
    8: "Mutual funds / ETFs",
    9: "Government bonds",
    10: "Indices",
    11: "Municipal bonds",
    12: "Not used",
    13: "Spread instruments",
    14: "Statistic symbols",
    15: "Monetary funds",
    16: "Unspecified bonds",
    17: "Certificates",
    18: "Warrants",
    19: "Money market symbols",
    20: "Forex symbols",
}


class SecurityTypeSpec:
    """ Morningstar Security Types

    Morningstar_Exchange_and_Field_Codes_v4.28.pdf - Page 25
    """

    @staticmethod
    def get_value(security_type: int) -> Optional[str]:
        return security_types.get(security_type, None)



from enum import Enum


class OnDemandReturnType(Enum):
    """ Morningstar Return Types in OnDemand

    Morningstar On Demand Webportal Dictionary - Return Types
    """
    TotalReturn = "1"
    LoadAdjustedReturn = "2"
    SECPreLiquidationReturn = "3"
    SECPostLiquidationReturn = "4"
    TaxCostRatioReturn_PreLiquidation = "5"
    MarketReturn = "6"
    IncomeReturn_PreTax = "7"
    CapitalGainReturn = "8"
    PremiumDiscountReturn = "9"
    IndexReturn = "10"
    TaxableEquivalentReturn = "11"
    NonStandardized_LoadAdjustedReturn = "12"
    GrossManagementFeeReturn = "13"
    SECPreLiquidationReturn_MarketPrice = "14"
    SECPostLiquidationReturn_MarketPrice = "15"
    TaxCostRatioReturn_PreLiquidation_MarketPrice = "16"
    PriceReturn = "17"
    TaxAdjustedReturn = "18"
    ExchangeRateReturn = "19"
    InvestorReturn = "20"
    NonAuditedReturn = "21"
    PreliminaryReturn = "31"
    EPRTotalReturn = "101"
    EPRLoadAdjustedReturn = "102"
    EPRNonStandardizedLoadAdjustedReturn = "112"
    PreInceptionTotalReturn = "201"
    PreInceptionLoadAdjustedReturn = "202"
    PreInceptionNASDNonStandardizedReturn = "212"
    SurvivorshipBiasFreeReturn = "301"
    TaxCostRatioReturn_PostLiquidation = "25"
    SECStandardizedReturn = "22"
    TaxAndLoadAdjustedReturn = "23"
    TaxAdjustedIncomeReturn = "27"
    


import unittest
import yaml
import logging

from morningstar.provider.morningstar import Morningstar

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

CONFIG_PATH = 'config-morningstar.yml.dist'
CONFIG = yaml.safe_load(open(CONFIG_PATH))['provider']['morningstar']
CONFIG_LIVE = None
try:
    from morningstar.config import config
    CONFIG_LIVE = config.get("provider")['morningstar']
except:
    logger.info("Skipping live tests.")
    pass


class MorningstarTest(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.provider = Morningstar(config=CONFIG)
        if CONFIG_LIVE is not None:
            self.provider_live = Morningstar(config=CONFIG_LIVE)

    def test_build_url(self):
        self.assertEqual(
            'endpoint?username=foo&password=bar&json',
            self.provider._build_url(base="endpoint", params={})
        )

    def test_build_url_params(self):
        self.assertEqual(
            'endpoint?username=foo&password=bar&isin=CH0038863350&json',
            self.provider._build_url(base="endpoint", params={'isin': 'CH0038863350'})
        )

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration missing")
    def test_search(self):
        r = self.provider_live.search({'isin': 'CH0038863350'})
        self.assertEqual([], r.errors)
        self.assertEqual(True, r.results[0].has_instrument())

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration required")
    def test_search_invalid(self):
        r = self.provider_live.search({'invalid_field': 'CH0038863350'})
        self.assertEqual(['Invalid request'], r.errors)

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration required")
    def test_search_empty(self):
        r = self.provider_live.search({'isin': 'nonexistent'})
        self.assertEqual([], r.errors)
        self.assertEqual([], r.results)

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration required")
    def test_index(self):
        r = self.provider_live.index({'instrument': '50.1.MSFT', 'fields': 'S9,D204'})
        self.assertEqual(
            {'Listed Currency': 'USD', 'Traded Currency': 'MXN'},
            r.results[0].data
        )

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration required")
    def test_index_invalid(self):
        r = self.provider_live.index({'instrument': '0.0.NONEXISTENT', 'fields': 'S9,D204'})
        self.assertEqual(['Invalid Instrument=0.0.NONEXISTENT'], r.errors)
        self.assertEqual([], r.results)

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration required")
    def test_indexts(self):
        r = self.provider_live.index_ts(
            {'instrument': '28.10.F00000JQA9', 'sdate': '01-01-2019', 'edate': '10-01-2019', 'type': 'dailybar'})
        self.assertEqual([], r.errors)
        self.assertEqual(7, len(r.results[0].data))

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration required")
    def test_indexts_no_data(self):
        r = self.provider_live.index_ts(
            {'instrument': '28.10.F00000JQA9', 'sdate': '01-01-2019', 'edate': '01-01-2019', 'type': 'dailybar'})
        self.assertEqual(['No Data'], r.errors)
        self.assertEqual(1, len(r.results))  # instrument is correct
        self.assertEqual(0, len(r.results.data))  # but doesnt contain data

    @unittest.skipIf(CONFIG_LIVE is None, "Live configuration required")
    def test_indexts_no_data(self):
        r = self.provider_live.index_ts(
            {'instrument': '0.0.NONEXISTENT', 'sdate': '01-01-2019', 'edate': '01-01-2019', 'type': 'dailybar'})
        self.assertEqual(['Invalid Instrument=0.0.NONEXISTENT'], r.errors)
        self.assertEqual([], r.results)



import unittest

from morningstar.spec.security_type_spec import SecurityTypeSpec


class SecurityTypeSpecTest(unittest.TestCase):

    def test_get_value(self):
        self.assertEqual("Stocks", SecurityTypeSpec.get_value(2))

    def test_get_value_none(self):
        self.assertIsNone(SecurityTypeSpec.get_value(200))



import logging

import requests

from morningstar.models.ms_response import MSResponse
from morningstar.models.ts_response import TSResponse
from morningstar.provider.provider import Provider

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class Morningstar(Provider):
    """Morningstar API

    Note:
        This class combines multiple endpoints:
            - http://msuxml.morningstar.com/IndexTS
            - http://msxml.tenfore.com/search
            - http://msxml.tenfore.com/index.php

    Attributes:
        credentials (dict): Provider specific configuration including "username" and "password"
    """

    def __init__(self, config):
        super().__init__(config)

    def _build_url(self, base: str, params: dict):
        url_params = ''.join(['&{}={}'.format(k, v) for k, v in params.items()])
        url = base + \
              '?username={}&password={}'.format(self.config['username'], self.config['password']) + \
              url_params + \
              '&json'
        return url

    def _request(self, base: str, params: dict):
        response = requests.get(self._build_url(base=base, params=params))
        return response.json()

    def _tenfore(self, endpoint: str, params: dict):
        base = 'http://msxml.tenfore.com/{}'.format(endpoint)
        return self._request(base=base, params=params)

    def _morningstar(self, endpoint: str, params: dict):
        base = 'http://msuxml.morningstar.com/{}'.format(endpoint)
        return self._request(base=base, params=params)

    def search(self, params):
        """Search endpoint

        Args:
            params (dict): e.g. {"isin": "US46625H1005"}

        Returns:

        """
        response = self._tenfore('search', params)
        return MSResponse.from_dict(response)

    def index(self, params):
        """Index endpoint

        Args:
            params (dict): e.g. {"isin": "US46625H1005"}

        Returns:

        """
        response = self._tenfore('index.php', params)
        return MSResponse.from_dict(response)

    def index_ts(self, params):
        """IndexTS endpoint

        Args:
            params (dict): e.g. {"isin": "US46625H1005"}

        Returns:

        """
        response = self._morningstar('IndexTS', params)
        return TSResponse.from_dict(response)



import codecs
import csv

import requests
from lxml import etree

from morningstar.models.rips import RIPS
from morningstar.provider.provider import Provider


class OnDemand(Provider):
    """OnDemand API

    Note:
        This class combines multiple endpoints:
            - https://edw.morningstar.com/login.aspx
            - https://edw.morningstar.com/Logout.aspx
            - https://edw.morningstar.com/GetUniverseXML.aspx
            - https://edw.morningstar.com/DataOutput.aspx
            - https://edw.morningstar.com/HistoryData/HistoryData.aspx

        Login needs to be called before using any of the other functions
        If logout is called, all clients currently accessing the API are logged out

    Attributes:
        credentials (dict): Provider specific configuration including "username" and "password"
    """

    url_login = "https://edw.morningstar.com/login.aspx"
    url_logout = "https://edw.morningstar.com/Logout.aspx"
    url_getuniverse = "https://edw.morningstar.com/GetUniverseXML.aspx"
    url_dataoutput = "https://edw.morningstar.com/DataOutput.aspx"
    url_historydata = "https://edw.morningstar.com/HistoryData/HistoryData.aspx"

    def __init__(self, config):
        super().__init__(config)
        self.session = None

    def login(self):
        self.session = requests.Session()
        ms_data_login = {
            'email': self.config['username'], 'password': self.config['password']}
        self.session.post(self.url_login, data=ms_data_login)

    def logout(self):
        self._assert_login()
        ms_data_logout = {'clientid': self.config['clientid']}
        self.session.post(url=self.url_logout, data=ms_data_logout)

    def _xml_from_url(self, url: str, params: dict):
        self._assert_login()
        r_url = self.session.get(url, params=params)
        return etree.fromstring(r_url.content)

    def get_universe(self, params: dict) -> [etree]:
        return self._xml_from_url(url=self.url_getuniverse, params=params)

    def data_output(self, params: dict) -> [etree]:
        return self._xml_from_url(url=self.url_dataoutput, params=params)

    def history_data(self, params: dict) -> [RIPS]:
        self._assert_login()
        with self.session.get(self.url_historydata, params=params) as r_download_csv:
            # read rows as dict to convert to rips class
            cr = csv.DictReader(codecs.iterdecode(
                r_download_csv.iter_lines(), 'utf-8'), delimiter=';')
            return [RIPS.from_dict(row_dict) for row_dict in cr]

    def _assert_login(self):
        if self.session is None:
            raise ValueError("A session has to be established first by calling login()")


import os
import logging
import yaml

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
CONFIG_FILE = "config-morningstar.yml"

config = None
for loc in os.path.join(os.curdir, CONFIG_FILE), \
           os.path.expanduser("~/{}".format(CONFIG_FILE)), \
           os.path.expanduser("~/.{}".format(CONFIG_FILE)):
    try:
        with open(loc) as source:
            config = yaml.safe_load(source)
    except IOError:
        pass
    except TypeError:
        pass

if config is None:
    logger.warning("{} not found.".format(CONFIG_FILE))



from morningstar.spec.web_service_spec import FieldNames


class Instrument:
    def __init__(self, exchange: str, security_type: str, symbol: str) -> None:
        self.exchange = exchange
        self.security_type = security_type
        self.symbol = symbol

    def __key(self):
        return self.exchange, self.security_type, self.symbol

    def __hash__(self):
        return hash(self.__key())

    def __eq__(self, other):
        return isinstance(self, type(other)) and self.__key() == other.__key()

    def __repr__(self):
        return '{}.{}.{}'.format(self.exchange, self.security_type, self.symbol)

    @staticmethod
    def from_string(s: str):
        l = s.split('.')
        if len(l) != 3:
            raise ValueError('Instrument not in format exchange.security_type.symbol')
        return Instrument(exchange=l[0], security_type=l[1], symbol=l[2])

    @staticmethod
    def from_dict(d: dict):
        return Instrument(
            exchange=d[FieldNames.Exchange.value],
            security_type=d[FieldNames.SecurityType.value],
            symbol=d[FieldNames.Symbol.value]
        )




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='morningstar',
    version='0.3.1',
    description='Morningstar API Client',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/aaaccell/morningstar',
    author='Marc Juchli',
    author_email='marcjuchli@aaaccell.ch',
    license='MIT',
    zip_safe=False,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



from morningstar.models.ts_result import TSResult


class TSResponse:
    def __init__(self, errors: list, results: list) -> None:
        self.errors = errors
        self.results = results

    @staticmethod
    def from_dict(d):
        ts = d['ts']
        errors = ts.get('error', [])
        results = ts.get('results', [])
        if results:
            results = [TSResult.from_json(r) for r in results]
        return TSResponse(errors=errors, results=results)
